/* extension.js
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 *
 * SPDX-License-Identifier: GPL-3.0-or-later
 */

/**
 * @author Ben
 * @see    https://github.com/andreabenini/gnome-plugin.custom-menu-panel
 */

const CONFIGURATION_FILE = '/.entries.json';

import Gio from 'gi://Gio';
import GLib from 'gi://GLib';
import GObject from 'gi://GObject';
import St from 'gi://St';

import {Extension, gettext as _} from 'resource:///org/gnome/shell/extensions/extension.js';

import * as Main      from 'resource:///org/gnome/shell/ui/main.js';
import * as BackgroundMenu from 'resource:///org/gnome/shell/ui/backgroundMenu.js';
import * as PanelMenu from 'resource:///org/gnome/shell/ui/panelMenu.js';
import * as PopupMenu from 'resource:///org/gnome/shell/ui/popupMenu.js';

import * as Config from './config.js';

const LOGGER_INFO       = 0;
const LOGGER_WARNING    = 1;
const LOGGER_ERROR      = 2;

const {BackgroundMenu: OriginalBackgroundMenu} = BackgroundMenu;


class CustomMenu extends PopupMenu.PopupMenu {
    constructor(sourceActor) {
        super(sourceActor, 0.0, St.Side.TOP, 0);

        this._loadSetup();
    }

    /**
     * LOAD Program settings from .entries.json file
     */
    _loadSetup() {
        this.removeAll();
        // Loading configuration from file
        this.configLoader = new Config.Loader();
        try {
            this.configLoader.loadConfig(GLib.get_home_dir() + CONFIGURATION_FILE);         // $HOME/.entries.json
        } catch(e) {
            this.configLoader.saveDefaultConfig(GLib.get_home_dir() + CONFIGURATION_FILE);  // create default entries
        }
        // Build the menu
        let i = 0;
        for (let i in this.configLoader.entries) {
            let item = this.configLoader.entries[i].createItem();
            this.addMenuItem(item);
        }
    } /**/
}

class CustomBackgroundMenu extends CustomMenu {
    constructor(layoutManager) {
        super(layoutManager.dummyCursor);

        this.actor.add_style_class_name('background-menu');

        layoutManager.uiGroup.add_actor(this.actor);
        this.actor.hide();

        this.connect('open-state-changed', (menu, open) => {
            if (open)
                this._updateMaxHeight();
        });
    }

    _updateMaxHeight() {
        const monitor = Main.layoutManager.findMonitorForActor(this.actor);
        const workArea = Main.layoutManager.getWorkAreaForMonitor(monitor.index);
        const {scaleFactor} = St.ThemeContext.get_for_stage(global.stage);
        const vMargins = this.actor.margin_top + this.actor.margin_bottom;
        const {y: offsetY} = this.sourceActor;

        const maxHeight = Math.round((monitor.height - offsetY - vMargins) / scaleFactor);
        this.actor.style = `max-height: ${maxHeight}px;`;
    }
}

const Indicator = GObject.registerClass(
    class Indicator extends PanelMenu.Button {
        _init() {
            super._init(0.0, _('Custom Menu Panel Indicator'), true);
            this.add_child(new St.Icon({
                icon_name: 'view-list-bullet-symbolic',
                style_class: 'system-status-icon',
            }));

            this.setMenu(new CustomMenu(this));
        }
    }
); /**/


class Logger {
    constructor(log_file) {
        this._log_file = log_file;
        // initailize log_backend
        if (!log_file) {
            this._initEmptyLog();
        } else if(log_file == "gnome-shell") {
            this._initGnomeLog();
        } else {
            this._initFileLog();
        }
        this.level = LOGGER_WARNING;
        this.info = function(t) {
            if (this.level <= LOGGER_INFO) {
                this.log(t);
            }
        };
        this.warning = function(t) {
            if (this.level <= LOGGER_WARNING) {
                this.log(t);
            }
        };
        this.error = function(t) {
            if (this.level <= LOGGER_ERROR) {
                this.log(t);
            }
        };
    }

    _initEmptyLog() {
        this.log = function(_) { };
    }

    _initGnomeLog() {
        this.log = function(s) {
            global.log("custom-menu-panel> " + s);
        };
    }

    _initFileLog() {
        this.log = function(s) {
            // all operations are synchronous: any needs to optimize?
            if (!this._output_file || !this._output_file.query_exists(null) || !this._fstream || this._fstream.is_closed()) {
                this._output_file = Gio.File.new_for_path(this._log_file);
                this._fstream = this._output_file.append_to(Gio.FileCreateFlags.NONE, null);
                if (!this._fstream instanceof Gio.FileIOStream) {
                    this._initGnomeLog();
                    this.log("IOError: Failed to append to " + this._log_file + " [Gio.IOErrorEnum:" + this._fstream + "]");
                    return;
                }
            }
            this._fstream.write(String(new Date())+" "+s+"\n", null);
            this._fstream.flush(null);
        }
    }

    notify(t, str, details) {
        this.ncond = this.ncond || ['proc', 'ext', 'state'];
        if (this.ncond.indexOf(t) < 0) {
            return;
        }
        Main.notify(str, details || "");
    }
}

// lazy-evaluation
let logger = null;
export function getLogger() {
    if (logger === null) {
        logger = new Logger("gnome-shell");
    }
    return logger;
} /**/

export default class CustomMenuExtension extends Extension {
    enable() {
        BackgroundMenu.BackgroundMenu = CustomBackgroundMenu;
        Main.layoutManager._updateBackgrounds();
    }

    disable() {
        BackgroundMenu.BackgroundMenu = OriginalBackgroundMenu;
        Main.layoutManager._updateBackgrounds();
    }
} /**/
